pub struct Messages<G, L> { /* private fields */ }Expand description
A message buffer of global messages (G) and local messages (L) meant for
consumption by clients. Local messages are those that have some spatial
component to them and implement the GetChunkPos trait. Local messages
are placed in a bounding volume hierarchy for fast queries via
Self::query_local. Global messages do not necessarily have a spatial
component and all globals will be visited when using Self::iter_global.
Every message is associated with an arbitrary span of bytes. The meaning of the bytes is whatever the message needs it to be.
At the end of the tick and before clients have access to the buffer, all messages are sorted and then deduplicated by concatenating byte spans together. This is done for a couple of reasons:
- Messages may rely on sorted message order for correctness, like in the case of entity spawn & despawn messages. Sorting also makes deduplication easy.
- Deduplication reduces the total number of messages that all clients must examine. Consider the case of a message such as “send all clients in view of this chunk position these packet bytes”. If two of these messages have the same chunk position, then they can just be combined together.
Implementations
Sourceimpl<G, L> Messages<G, L>
impl<G, L> Messages<G, L>
Sourcepub fn bytes(&self) -> &[u8] ⓘ
pub fn bytes(&self) -> &[u8] ⓘ
All message bytes. Use this in conjunction with Self::iter_global
and Self::query_local.
Sourcepub fn iter_global(&self) -> impl Iterator<Item = (G, Range<usize>)>
pub fn iter_global(&self) -> impl Iterator<Item = (G, Range<usize>)>
Returns an iterator over all global messages and their span of bytes in
Self::bytes.
Sourcepub fn query_local<F>(&self, view: ChunkView, f: F)
pub fn query_local<F>(&self, view: ChunkView, f: F)
Takes a visitor function f and visits all local messages contained
within the chunk view view. f is called with the local
message and its span of bytes in Self::bytes.